#include "SiteServer.h"

#include "Sensors/TemperatureSensor.h"

#include <functional>
#include <memory>

namespace Passer {
namespace Control {

SiteServer::SiteServer(int port) {
  this->name = "Site Server";
  this->publishInterval = 0;

  this->ipAddress = "0.0.0.0";
  this->port = port;

  this->participants.push_back(this);

  SetupUDP(port, ipAddress, 0);

  Register<TemperatureSensor>((unsigned char)Thing::Type::TemperatureSensor);
}

void SiteServer::Process(Participant *sender, ClientMsg *msg) {
  if (msg->networkId == 0) {
    std::cout << this->name << " received New Client -> " << sender->ipAddress
              << "  " << (int)sender->networkId << "\n";
    NetworkIdMsg *msg = new NetworkIdMsg(sender->networkId);
    sender->Send(msg);
    delete msg;
  }
}

void SiteServer::Process(Participant *sender, NetworkIdMsg *msg) {}

void SiteServer::Process(ThingMsg *msg) {

  Thing *thing = this->Get(msg->networkId, msg->thingId);
  if (thing == nullptr) {
    std::cout << "could not find thing " << (int)msg->networkId << "/"
              << (int)msg->thingId << "\n";
    auto thingMsgProcessor = thingMsgProcessors.find(msg->thingType);
    if (thingMsgProcessor != thingMsgProcessors.end()) { // found item
      Thing *newThing = thingMsgProcessor->second(msg->networkId, msg->thingId);
      this->Add(newThing);
    } else {
      Thing *newThing = new Thing(this, msg->networkId, msg->thingId,
                                  (Thing::Type)msg->thingType);
      this->Add(newThing);
    }
  }
}

} // namespace Control
} // namespace Passer
